@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+
 module TextBuilderDev
   ( TextBuilder,
 
@@ -71,6 +73,8 @@ module TextBuilderDev
     utcTimeInIso8601,
     utcTimestampInIso8601,
     intervalInSeconds,
+    diffTimeCompact,
+    picosecondsCompact,
 
     -- * Classes
     IsomorphicToTextBuilder (..),
@@ -88,6 +92,10 @@ import qualified Test.QuickCheck.Gen as QcGen
 import qualified TextBuilderDev.Allocator as Allocator
 import TextBuilderDev.Prelude hiding (intercalate, length, null)
 
+#if MIN_VERSION_text(2,0,2)
+import qualified Data.Text.Encoding as TextEncoding
+#endif
+
 -- * --
 
 -- |
@@ -138,6 +146,12 @@ instance IsomorphicToTextBuilder TextLazyBuilder.Builder where
   toTextBuilder = text . TextLazy.toStrict . TextLazyBuilder.toLazyText
   fromTextBuilder = TextLazyBuilder.fromText . buildText
 
+#if MIN_VERSION_text(2,0,2)
+instance IsomorphicToTextBuilder TextEncoding.StrictBuilder where
+  toTextBuilder = toTextBuilder . TextEncoding.strictBuilderToText
+  fromTextBuilder = TextEncoding.textToStrictBuilder . fromTextBuilder
+#endif
+
 -- * --
 
 -- |
@@ -196,8 +210,8 @@ instance Arbitrary TextBuilder where
         finiteBitsUnsignedBinary @Word <$> arbitrary,
         hexadecimal @Integer <$> arbitrary,
         unsignedHexadecimal @Natural <$> arbitrary,
-        decimalDigit <$> QcGen.choose (0, 9),
-        hexadecimalDigit <$> QcGen.choose (0, 15),
+        decimalDigit <$> QcGen.choose @Int (0, 9),
+        hexadecimalDigit <$> QcGen.choose @Int (0, 15),
         fixedDouble <$> QcGen.choose (0, 19) <*> arbitrary,
         doublePercent <$> QcGen.choose (0, 19) <*> arbitrary,
         utcTimestampInIso8601 <$> arbitrary <*> arbitrary <*> arbitrary <*> arbitrary <*> arbitrary <*> arbitrary,
@@ -481,14 +495,14 @@ unsignedHexadecimal =
 
 -- | Decimal digit.
 {-# INLINE decimalDigit #-}
-decimalDigit :: Int -> TextBuilder
-decimalDigit n =
+decimalDigit :: (Integral a) => a -> TextBuilder
+decimalDigit (fromIntegral -> n) =
   unicodeCodePoint (n + 48)
 
 -- | Hexadecimal digit.
 {-# INLINE hexadecimalDigit #-}
-hexadecimalDigit :: Int -> TextBuilder
-hexadecimalDigit n =
+hexadecimalDigit :: (Integral a) => a -> TextBuilder
+hexadecimalDigit (fromIntegral -> n) =
   if n <= 9
     then unicodeCodePoint (n + 48)
     else unicodeCodePoint (n + 87)
@@ -583,6 +597,11 @@ utcTimestampInIso8601 y mo d h mi s =
 
 -- |
 -- Time interval in seconds.
+--
+-- The format is the following:
+--
+-- > DD:HH:MM:SS
+--
 -- Directly applicable to 'DiffTime' and 'NominalDiffTime'.
 {-# INLINEABLE intervalInSeconds #-}
 intervalInSeconds :: (RealFrac seconds) => seconds -> TextBuilder
@@ -600,6 +619,31 @@ intervalInSeconds interval = flip evalState (round interval) $ do
     <> ":"
     <> padFromLeft 2 '0' (decimal seconds)
 
+-- | DiffTime in a compact decimal format based on 'picosecondsCompact'.
+diffTimeCompact :: DiffTime -> TextBuilder
+diffTimeCompact = picosecondsCompact . diffTimeToPicoseconds
+
+-- | Amount of picoseconds represented in a compact decimal format using suffixes.
+--
+-- E.g., the following is @1_230_000_000@ picoseconds or 1.23 milliseconds or 1230 microseconds:
+--
+-- > 1230us
+picosecondsCompact :: Integer -> TextBuilder
+picosecondsCompact x =
+  attemptOr 1_000_000_000_000 "s"
+    $ attemptOr 1_000_000_000 "ms"
+    $ attemptOr 1_000_000 "us"
+    $ attemptOr 1_000 "ns"
+    $ decimal x
+    <> "ps"
+  where
+    attemptOr factor suffix alternative =
+      if x == divided * factor
+        then decimal divided <> suffix
+        else alternative
+      where
+        divided = div x factor
+
 -- | Double with a fixed number of decimal places.
 {-# INLINE fixedDouble #-}
 fixedDouble ::
